clear all;
clc;

% format float with SI prefixes
function [str] = fmt_si(val,dig,spc)
  units = {'y','z','a','f','p','n','u','m','','k','M','G','T','P','E','Z','Y'};
  lim = 1e-24*(1000.^((1:length(units))-1));
  id = find(lim<=val,1,'last');
  if(isempty(id))
    id = 5;
  end
  if(~exist('spc','var'))
    spc = '';
  end  
  str = [num2str(val/lim(id),dig) spc units{id}];
endfunction

% set current directory (new Octave can't read data from path set by --path command line argument)
mfld = mfilename('fullpath'); mfld = mfld(1:strchr(mfld,'\',1,'last'));
cd(mfld);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This script is part of experiment with AVR ADC converter in equivalent 
%% time sampling mode. The idea is based on sampling above Nyquist sampling 
%% criterium fs/2 which generates alias signal. This is usually unwanted effect 
%% but in case sampled signal is periodical and has certain frequency ratio to 
%% sampling rate, it can be then used for time scale magnification.
%% This experiment assumes usage of internal AVR ADC clocked at certain rate 
%% and generation of sampled signal by some of AVR timers. This way both 
%% frequencies, the sampling rate and fundametal freq. of the sampled signal, 
%% are derived from common clock source and thus it is possible to reach stable 
%% ratios. For some ratios we'll get aliases with integer count of samples per 
%% sampled signal period. Following script can be used to find such ratios.
%% I successfully used this technique to measure true-rms value of current in 
%% AVR PWM channel driven switch mode PSU.
%%
%% Meaning of the output:
%%  f = frequency of generated (and sampled) signal
%%  total = total samples count per period of sampled signal
%%  subs = number of interleaved subsets per signal period
%%  fs' = equivalent sampling frequency
%%  acq_t = total acquisition time of single period of signal
%% 
%% Example:
%% --------
%% XTAL = 10 MHz
%% ADC in free running mode and clock /128:
%%  - clock = 78.125 kHz
%%  - real time sample rate = 6.009615 ks/s
%% Timer in function of signal generator:
%%  - clock div. = 8
%%  - OCR = 91
%%  - PWM @ 50% mode output freq. = 13.5870 kHz
%% Total (equivalent) samples per period = 23
%% Interleaved subsets = 6
%% Equivalent sampling rate = 312.5 ks/s
%%
%% In this case the samples acquired by ADC comes in order:
%% w = [0 6 12 18 1 7 13 19 2 8 14 20 3 9 15 21 4 10 16 22 5 11 17]
%% After reordering these, which can be easily done runtime in ADC ISR, 
%% we have waveform w with one period of our signal at equivalent sampling 
%% rate 312.5 ks/s (23*13.587...).
%% 
%% The runtime reordering can be done by following code (AVRGCC):
%% 
%% // round buffer
%% uint16_t adc_buf[total];
%%
%% ISR(ADC_vect)
%% {
%%	// round buffer pointer
%%	static uint16_t *p = &adc_buf[0];
%%
%%	// store new sample 
%%	*p++ = ADC;
%%	// update address
%%	p += (subs-1);
%%	// round buffer overflow?
%%	if(p >= &adc_buf[total])
%%		p -= total; // yaha, start from beginning
%%
%%	adc_buf_p = p;
%% }
%%
%%
%% Version 2013.08.01 by Stanislav Maslan, s.maslan@seznam.cz
%% http://elektronika.kvalitne.cz/ATMEL/necoteorie/AVR_ETS_bandwidth/AVR_ETS_bandwidth.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  

%%% INPUTS ***
% xtal frequency [Hz]
f_xtal = 20e6;

% ADC clock divider
adc_div = 128;
% ADC clock cycles per conversion (13 for free-run)
adc_cycles = 13;

% signal timer: clock divider
tmr_div = 1;
% signal timer: max count
tmr_max = 1000;
% signal timer: timer periods per signal period (2 for toggle, 1 for 50% PWM mode)
tmr_oc_per = 1; 

% signal frequency range [Hz]
sig_f_rng = [20e+3,50e3];
% max equivalent samples per signal period
sig_max_smpl = 2000;
% max interleaved subsets per signal period
sig_max_subs = sig_max_smpl;


%%% CALCULATION %%%
% ADC clock
adc_ck = f_xtal/adc_div; 
% ADC real time sampling rate
adc_fs = adc_ck/adc_cycles;

% OCRs list
tmr_ocrs = tmr_max:-1:1;
% OC frequencies
tmr_f = f_xtal/tmr_div/tmr_oc_per./(tmr_ocrs+1);
% reduce result into valid freq. range
ids = find(tmr_f>=min(sig_f_rng) & tmr_f<=max(sig_f_rng));
tmr_ocrs = tmr_ocrs(ids);
tmr_f = tmr_f(ids);

% frequency ratios 
r = 1./mod(tmr_f/adc_fs,1);

% create log file
fw = fopen('divisors_log.txt','w');

%% find ratios for integer sample count per period
for k = 1:length(r) 
  % skip for invalids
  if(r(k) == Inf)
    continue;
  end
  
  % try to find number of interleaved subsets for integer sample count per signal period 
  sub = find(mod((1:min(sig_max_smpl,sig_max_subs))*r(k),1)<1e-10,1);
  
  % total samples per signal period
  tot = r(k)*sub;
  
  % equivalent samplign frequency
  fs_eq = tmr_f(k)*tot;
  
  % acquisition time of single period of the signal
  acq_t = tot/adc_fs;
  
  % skip if nothing found or invalid
  if(isempty(sub) || tot>sig_max_smpl || sub>sig_max_subs)
    continue;
  end
  
  % printf result
  str = sprintf('OCR = %5d: total = %4d, subs = %4d, f = %sHz, fs'' = %-14s, acq_t = %ss\n',tmr_ocrs(k),tot,sub,fmt_si(tmr_f(k),'%6.4f',' '),[fmt_si(fs_eq,8,' ') 's/s'],fmt_si(acq_t,5,' '));  
  puts(str);
  fputs(fw,str);  
end

% prinf ADC setup
str = sprintf('\nADC clock = %sHz\nADC real time sampling rate = %sHz\n',fmt_si(adc_ck,8,' '),fmt_si(adc_fs,8,' '));
puts(str);
fputs(fw,str);

% close log file
fclose(fw);




